package com.hikcreate.drv_photo_pic.impl;

import com.hikcreate.drv_photo_pic.Vehicle;
import com.hikcreate.entity.PicResult;
import com.hikcreate.service.fdfs.service.FileService;
import com.hikcreate.utils.DateUtil;
import oracle.sql.BLOB;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.*;
import java.sql.Connection;

@Service("vehicleImpl")
public class VehicleImpl implements Vehicle {

    private static Logger logger = LoggerFactory.getLogger(VehicleImpl.class);
    @Autowired
    private FileService fileService;

    @Value("${hbase.veh_pic.table}")
    private String vehPicTableStr;
    @Value("${hbase.zookeeper.property.clientPort}")
    private String hbaseAddress;
    @Value("${hbase.zookeeper.quorum}")
    private String hbaseZkQuorum;
    @Value("${url}")
    private String url;
    @Value("${username}")
    private String username;
    @Value("${password}")
    private String password;
    @Value("${roundDay}")
    private int roundDay;

    private static String startTime;
    private static String endTime;
    private static TableName vehPicTableName;
    private static Table vehPicTable;
    private static String[] colValue;

    private static Configuration hbaseConf;
    private static org.apache.hadoop.hbase.client.Connection hbaseConn;

    /**
     * 全量车辆图片
     *
     * @param time
     */
    @Override
    public void getHisVehPic(String time) {
        getHbaseVioPicTable();
        try {
            endTime = time;
            startTime = SqlHelp.getStartTime("select  min(GXSJ) mtime from GYJG.VEH_PICTURE", url, username, password);
            String lastTime;
            String sql;
            while (endTime.compareTo(startTime) > 0 || endTime.compareTo(startTime) == 0) {
                Connection connection = DriverManager.getConnection(url, username, password);
                lastTime = DateUtil.formatDate(DateUtil.getPastDate(DateUtil.parseDate(endTime), roundDay));
                sql = "select * from GYJG.VEH_PICTURE where to_char(GXSJ,'yyyy-mm-dd')>=? and to_char(GXSJ,'yyyy-mm-dd')<? " +
                        " and ZP is not null and XH is not null ";
                PreparedStatement pstm = connection.prepareStatement(sql);
                pstm.setString(1, lastTime);
                pstm.setString(2, endTime);
                ResultSet resultSet = pstm.executeQuery();
                while (resultSet.next()) {
                    saveToHbase(resultSet, fileService);
                }
                endTime = lastTime;
                resultSet.close();
                connection.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            closeHbase();
        }
    }

    /**
     * 增量车辆图片
     */
    @Override
    public void getIncrementVehPic() {
        String sql="select * from GYJG.VEH_PICTURE where GXSJ between sysdate-10 and sysdate and ZP is not null and XH is not null";
        getPic(sql);
    }

    /**
     * 根据车牌号获取图片
     * @param hphm
     * @param hpzl
     */
    @Override
    public void insertVehPicByCar(String hphm, String hpzl) {
        String sql = "select * from GYJG.VEH_PICTURE where HPHM='"+hphm+"' and HPZL='"+hpzl+"'";
        getPic(sql);
    }

    /**
     * 同步指定时间段的增量图片
     * @param start
     * @param end
     */
    @Override
    public void getVehPicStage(String start, String end) {
        String sql="select * from GYJG.VEH_PICTURE where " +
                "to_char(GXSJ,'yyyy-mm-dd')>'"+start+"' and to_char(GXSJ,'yyyy-mm-dd')<='"+end+"' and ZP is not null and XH is not null";
        getPic(sql);
    }

    /**
     * 根据序号插入图片
     * @param xh
     */
    @Override
    public void insertVehPicByXh(String xh) {
        String sql = "select * from GYJG.VEH_PICTURE where XH='"+xh+"'";
        getPic(sql);
    }


    private void getPic(String sql) {
        getHbaseVioPicTable();
        try {
            Class.forName("oracle.jdbc.OracleDriver");
            Connection connection = DriverManager.getConnection(url, username, password);
            PreparedStatement pstm = connection.prepareStatement(sql);
            ResultSet resultSet=pstm.executeQuery();
            while (resultSet.next()) {
                saveToHbase(resultSet, fileService);
            }
        }catch (Exception e) {
            e.printStackTrace();
        } finally {
            closeHbase();
        }
    }


    /**
     * 关闭hbase
     */
    private void closeHbase() {
        if (hbaseConn != null) {
            try {
                hbaseConn.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (vehPicTable != null) {
            try {
                vehPicTable.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 存储图片
     *
     * @param resultSet
     * @param fileService
     */
    private synchronized void saveToHbase(ResultSet resultSet, FileService fileService) {
        try {
            String xh = resultSet.getString("XH");
            String hphm = resultSet.getString("HPHM");
            String hpzl = resultSet.getString("HPZL");
            BLOB zp = (BLOB) resultSet.getBlob("ZP");
            String gxsj = resultSet.getString("GXSJ");
            String rowkey = xh;
            colValue = new String[2];
            PicResult picResult = SqlHelp.getFlag(rowkey, vehPicTable, colValue, gxsj);
            if (picResult.getI()!=0) {
                if(picResult.getI()==2){
                    fileService.deleteFile(picResult.getUrl());
                }
                Put put = new Put(rowkey.getBytes());
                put.addColumn("info".getBytes(), "xh".getBytes(), (xh == null ? "null" : xh).getBytes());
                put.addColumn("info".getBytes(), "hphm".getBytes(), (hphm == null ? "null" : hphm).getBytes());
                put.addColumn("info".getBytes(), "hpzl".getBytes(), (hpzl == null ? "null" : hpzl).getBytes());
                put.addColumn("info".getBytes(), "gxsj".getBytes(), (gxsj == null ? "null" : gxsj).getBytes());
                byte[] bytes = SqlHelp.blobToByteZp(zp);
                String picUrl = fileService.uploadFile(bytes, xh + ".jpeg");
                put.addColumn("info".getBytes(), "picUrl".getBytes(), picUrl.getBytes());
                vehPicTable.put(put);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    private void getHbaseVioPicTable() {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
        hbaseConf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
        try {
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            vehPicTableName = TableName.valueOf(vehPicTableStr);
            vehPicTable = hbaseConn.getTable(vehPicTableName);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
